@@ -1,7 +1,7 @@
 # FileFormat readers must be imported here
 
 # Meta-readers to handle extension conflicts
-from .meta import DatMetaReader, HDRMetaReader
+from .meta import DatMetaReader, HDRMetaReader, HDF5MetaReader
 
 # General readers
 from .ascii import AsciiColReader, AsciiMapReader
@@ -12,7 +12,7 @@ from .matlab import MatlabReader
 # Instrument-specific readers
 from .agilent import AgilentImageReader, AgilentImageIFGReader, agilentMosaicReader,\
     agilentMosaicIFGReader, agilentMosaicTileReader
-from .neaspec import NeaReader, NeaReaderGSF
+from .neaspec import NeaReader, NeaReaderGSF, NeaReaderMultiChannel
 from .omnic import OmnicMapReader, SPAReader, SPCReader
 from .opus import OPUSReader
 from .ptir import PTIRFileReader
@@ -22,3 +22,4 @@ from .wire import WiREReaders
 from .diamond import NXS_STXM_Diamond_I08
 from .maxiv import HDRReader_STXM
 from .soleil import SelectColumnReader, HDF5Reader_HERMES, HDF5Reader_ROCK
+from .cls import HDF5Reader_SGM
