@@ -7,14 +7,23 @@ from base64 import b64decode
 import numpy as np
 from PIL import Image
 
+try:
+    import dask
+    from Orange.tests.test_dasktable import temp_dasktable
+except ImportError:
+    dask = None
+
 from AnyQt.QtCore import QPointF, Qt, QRectF
 from AnyQt.QtTest import QSignalSpy
 import Orange
 from Orange.data import DiscreteVariable, Domain, Table
 from Orange.widgets.tests.base import WidgetTest
+from Orange.util import OrangeDeprecationWarning
 
 from orangecontrib.spectroscopy.data import _spectra_from_image, build_spec_table
-from orangecontrib.spectroscopy.preprocess.integrate import IntegrateFeaturePeakSimple, Integrate
+from orangecontrib.spectroscopy.io.util import VisibleImage
+from orangecontrib.spectroscopy.preprocess.integrate import IntegrateFeaturePeakSimple, \
+    Integrate, IntegrateFeatureSimple
 from orangecontrib.spectroscopy.widgets import owhyper
 from orangecontrib.spectroscopy.widgets.owhyper import \
     OWHyper
@@ -113,11 +122,19 @@ class TestOWHyper(WidgetTest):
         # dataset without rows
         empty = cls.iris[:0]
         # dataset with large blank regions
-        irisunknown = Interpolate(np.arange(20))(cls.iris)
+        irisunknown = Interpolate(np.arange(20))(cls.iris)[:20]
         # dataset without any attributes, but XY
         whitelight0 = cls.whitelight.transform(
-            Orange.data.Domain([], None, metas=cls.whitelight.domain.metas))
-        cls.strange_data = [None, cls.iris1, iris0, empty, irisunknown, whitelight0]
+            Orange.data.Domain([], None, metas=cls.whitelight.domain.metas))[:100]
+        unknowns = cls.iris[::10].copy()
+        with unknowns.unlocked():
+            unknowns.X[:, :] = float("nan")
+        single_pixel = cls.iris[:50]  # all image coordinates map to one spot
+        unknown_pixel = single_pixel.copy()
+        with unknown_pixel.unlocked():
+            unknown_pixel.Y[:] = float("nan")
+        cls.strange_data = [None, cls.iris1, iris0, empty, irisunknown, whitelight0,
+                            unknowns, single_pixel, unknown_pixel]
 
     def setUp(self):
         self.widget = self.create_widget(OWHyper)  # type: OWHyper
@@ -159,7 +176,9 @@ class TestOWHyper(WidgetTest):
 
     def test_strange(self):
         for data in self.strange_data:
+            self.widget = self.create_widget(OWHyper)
             self.send_signal("Data", data)
+            wait_for_image(self.widget)
             self.try_big_selection()
 
     def test_context_not_open_invalid(self):
@@ -185,9 +204,11 @@ class TestOWHyper(WidgetTest):
         self.assertIsNone(self.get_output("Selection"), None)
 
     def test_unknown(self):
-        self.send_signal("Data", self.whitelight)
+        self.send_signal("Data", self.whitelight[:10])
+        wait_for_image(self.widget)
         levels = self.widget.imageplot.img.levels
-        self.send_signal("Data", self.whitelight_unknown)
+        self.send_signal("Data", self.whitelight_unknown[:10])
+        wait_for_image(self.widget)
         levelsu = self.widget.imageplot.img.levels
         np.testing.assert_equal(levelsu, levels)
 
@@ -214,9 +235,11 @@ class TestOWHyper(WidgetTest):
         self.assertIsNone(out, None)
 
         # select specific points
-        self.widget.imageplot.select_square(QPointF(9.4, 9.4), QPointF(11.6, 10.6))
+        self.widget.imageplot.select_square(QPointF(53.20, 30.0), QPointF(53.21, 30.03))
         out = self.get_output("Selection")
-        np.testing.assert_equal(out.metas, [[10, 10], [11, 10]])
+        np.testing.assert_almost_equal(out.metas,
+                                       [[53.2043, 30.0185], [53.2043, 30.0085]],
+                                       decimal=3)
         np.testing.assert_equal([o[out.domain["Group"]].value for o in out],
                                 ["G1", "G1"])
 
@@ -224,40 +247,43 @@ class TestOWHyper(WidgetTest):
         # rectangle and a polygon need to give the same results
         self.send_signal("Data", self.whitelight)
         wait_for_image(self.widget)
-        self.widget.imageplot.select_square(QPointF(5, 5), QPointF(15, 10))
+        self.widget.imageplot.select_square(QPointF(53, 30), QPointF(54, 31))
         out = self.get_output("Selection")
-        self.widget.imageplot.select_polygon([QPointF(5, 5), QPointF(15, 5), QPointF(15, 10),
-                                              QPointF(5, 10), QPointF(5, 5)])
+        self.widget.imageplot.select_polygon([QPointF(53, 30), QPointF(53, 31), QPointF(54, 31),
+                                              QPointF(54, 30), QPointF(53, 30)])
         outpoly = self.get_output("Selection")
         self.assertEqual(list(out), list(outpoly))
 
     def test_select_click(self):
         self.send_signal("Data", self.whitelight)
         wait_for_image(self.widget)
-        self.widget.imageplot.select_by_click(QPointF(1, 2))
+        self.widget.imageplot.select_by_click(QPointF(53.2443, 30.6984))
         out = self.get_output("Selection")
-        np.testing.assert_equal(out.metas, [[1, 2]])
+        np.testing.assert_almost_equal(out.metas, [[53.2443, 30.6984]], decimal=3)
 
     def test_select_click_multiple_groups(self):
         data = self.whitelight
         self.send_signal("Data", data)
         wait_for_image(self.widget)
-        self.widget.imageplot.select_by_click(QPointF(1, 2))
+        self.widget.imageplot.select_by_click(QPointF(53.2, 30))
         with hold_modifiers(self.widget, Qt.ShiftModifier):
-            self.widget.imageplot.select_by_click(QPointF(2, 2))
+            self.widget.imageplot.select_by_click(QPointF(53.4, 30))
         with hold_modifiers(self.widget, Qt.ShiftModifier):
-            self.widget.imageplot.select_by_click(QPointF(3, 2))
+            self.widget.imageplot.select_by_click(QPointF(53.6, 30))
         with hold_modifiers(self.widget, Qt.ControlModifier):
-            self.widget.imageplot.select_by_click(QPointF(4, 2))
+            self.widget.imageplot.select_by_click(QPointF(53.8, 30))
         out = self.get_output(self.widget.Outputs.annotated_data)
         self.assertEqual(len(out), 20000)  # have a data table at the output
         newvars = out.domain.variables + out.domain.metas
         oldvars = data.domain.variables + data.domain.metas
         group_at = [a for a in newvars if a not in oldvars][0]
         unselected = group_at.to_val("Unselected")
-        out = out[np.flatnonzero(out.transform(Orange.data.Domain([group_at])).X != unselected)]
+        out = out[np.asarray(np.flatnonzero(
+            out.transform(Orange.data.Domain([group_at])).X != unselected))]
         self.assertEqual(len(out), 4)
-        np.testing.assert_equal([o["map_x"].value for o in out], [1, 2, 3, 4])
+        np.testing.assert_almost_equal([o["map_x"].value for o in out],
+                                       [53.1993, 53.3993, 53.5993, 53.7993],
+                                       decimal=3)
         np.testing.assert_equal([o[group_at].value for o in out], ["G1", "G2", "G3", "G3"])
         out = self.get_output(self.widget.Outputs.selected_data)
         np.testing.assert_equal([o[out.domain["Group"]].value for o in out],
@@ -265,7 +291,7 @@ class TestOWHyper(WidgetTest):
 
         # remove one element
         with hold_modifiers(self.widget, Qt.AltModifier):
-            self.widget.imageplot.select_by_click(QPointF(1, 2))
+            self.widget.imageplot.select_by_click(QPointF(53.2, 30))
         out = self.get_output(self.widget.Outputs.selected_data)
         np.testing.assert_equal(len(out), 3)
         np.testing.assert_equal([o[out.domain["Group"]].value for o in out],
@@ -284,7 +310,7 @@ class TestOWHyper(WidgetTest):
         self.assertEqual(self.widget.curveplot.feature_color.name, "iris")
 
     def test_set_variable_color(self):
-        data = Orange.data.Table("iris.tab")
+        data = self.iris
         ndom = Orange.data.Domain(data.domain.attributes[:-1], data.domain.class_var,
                                   metas=[data.domain.attributes[-1]])
         data = data.transform(ndom)
@@ -359,7 +385,7 @@ class TestOWHyper(WidgetTest):
             self.assertGreater(os.path.getsize(fname), 1000)
 
     def test_unknown_values_axes(self):
-        data = Orange.data.Table("iris")
+        data = self.iris.copy()
         with data.unlocked():
             data.Y[0] = np.nan
         self.send_signal("Data", data)
@@ -367,12 +393,14 @@ class TestOWHyper(WidgetTest):
         self.assertTrue(self.widget.Information.not_shown.is_shown())
 
     def test_migrate_context_feature_color(self):
-        c = self.widget.settingsHandler.new_context(self.iris.domain,
-                                                    None, self.iris.domain.class_vars)
+        # avoid class_vars in tests, because the setting does not allow them
+        iris = self.iris.transform(
+            Domain(self.iris.domain.attributes, None, self.iris.domain.class_vars))
+        c = self.widget.settingsHandler.new_context(iris.domain, None, iris.domain.metas)
         c.values["curveplot"] = {"feature_color": ("iris", 1)}
         self.widget = self.create_widget(OWHyper,
                                          stored_settings={"context_settings": [c]})
-        self.send_signal("Data", self.iris)
+        self.send_signal("Data", iris)
         self.assertIsInstance(self.widget.curveplot.feature_color, DiscreteVariable)
 
     def test_image_computation(self):
@@ -392,7 +420,9 @@ class TestOWHyper(WidgetTest):
 
         wrap = self.widget.imageplot.img
 
-        # integrals from zero; default
+        # integral from zero
+        self.widget.integration_method = \
+            self.widget.integration_methods.index(IntegrateFeatureSimple)
         self.send_signal("Data", data)
         with patch.object(wrap, 'setImage', wraps=wrap.setImage) as m:
             wait_for_image(self.widget)
@@ -462,10 +492,37 @@ class TestOWHyper(WidgetTest):
         self.assertTrue(self.widget.compat_no_group)
 
 
+
+@unittest.skipUnless(dask, "installed Orange does not support dask")
+class TestOWHyperWithDask(TestOWHyper):
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.iris = temp_dasktable("iris")
+        cls.whitelight = temp_dasktable("whitelight.gsf")
+        cls.whitelight_unknown = temp_dasktable(cls.whitelight_unknown)
+        cls.iris1 = temp_dasktable(cls.iris1)
+        cls.strange_data = [temp_dasktable(d) if d is not None else None
+                            for d in cls.strange_data]
+
+
+class _VisibleImageStream(VisibleImage):
+    """ Do not use this class in practice because too many things
+    will get copied when transforming tables."""
+
+    def __init__(self, name, pos_x, pos_y, size_x, size_y, stream):
+        super().__init__(name, pos_x, pos_y, size_x, size_y)
+        self.stream = stream
+
+    @property
+    def image(self):
+        return Image.open(self.stream)
+
+
 class TestVisibleImage(WidgetTest):
 
     @classmethod
-    def mock_visible_image_data(cls):
+    def mock_visible_image_data_oldformat(cls):
         red_img = io.BytesIO(b64decode("iVBORw0KGgoAAAANSUhEUgAAAA"
                                        "oAAAAKCAYAAACNMs+9AAAAFUlE"
                                        "QVR42mP8z8AARIQB46hC+ioEAG"
@@ -502,6 +559,23 @@ class TestVisibleImage(WidgetTest):
             },
         ]
 
+    @classmethod
+    def mock_visible_image_data(cls):
+        red_img = io.BytesIO(b64decode("iVBORw0KGgoAAAANSUhEUgAAAA"
+                                       "oAAAAKCAYAAACNMs+9AAAAFUlE"
+                                       "QVR42mP8z8AARIQB46hC+ioEAG"
+                                       "X8E/cKr6qsAAAAAElFTkSuQmCC"))
+        black_img = io.BytesIO(b64decode("iVBORw0KGgoAAAANSUhEUgAAA"
+                                         "AoAAAAKCAQAAAAnOwc2AAAAEU"
+                                         "lEQVR42mNk+M+AARiHsiAAcCI"
+                                         "KAYwFoQ8AAAAASUVORK5CYII="))
+
+        return [
+            _VisibleImageStream("Image 01", 100, 100, 17., 23., red_img),
+            _VisibleImageStream("Image 02", 0.5, 0.5, 10, 3, black_img),
+            _VisibleImageStream("Image 03", 100, 100, 17., 23., red_img),
+        ]
+
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
@@ -515,11 +589,10 @@ class TestVisibleImage(WidgetTest):
         self.widget = self.create_widget(OWHyper)  # type: OWHyper
 
     def assert_same_visible_image(self, img_info, vis_img, mock_rect):
-        img = Image.open(img_info["image_ref"]).convert('RGBA')
+        img = img_info.image.convert('RGBA')
         img = np.array(img)[::-1]
-        rect = QRectF(img_info['pos_x'], img_info['pos_y'],
-                      img.shape[1] * img_info['pixel_size_x'],
-                      img.shape[0] * img_info['pixel_size_y'])
+        rect = QRectF(img_info.pos_x, img_info.pos_y,
+                      img_info.size_x, img_info.size_y)
         self.assertTrue((vis_img.image == img).all())
         mock_rect.assert_called_with(rect)
 
@@ -665,3 +738,23 @@ class TestVisibleImage(WidgetTest):
             self.assert_same_visible_image(data.attributes["visible_images"][0],
                                            w.imageplot.vis_img,
                                            mock_rect)
+
+    def test_oldformat(self):
+        data = Orange.data.Table(
+            "agilent/4_noimage_agg256.dat"
+        )
+        data.attributes["visible_images"] = \
+            self.mock_visible_image_data_oldformat()
+        w = self.widget
+
+        with self.assertWarns(OrangeDeprecationWarning):
+            self.send_signal("Data", data)
+
+        wait_for_image(w)
+
+        self.assertNotIn(w.imageplot.vis_img, w.imageplot.plot.items)
+
+        w.controls.show_visible_image.setChecked(True)
+        self.assertIn(w.imageplot.vis_img, w.imageplot.plot.items)
+        w.controls.show_visible_image.setChecked(False)
+        self.assertNotIn(w.imageplot.vis_img, w.imageplot.plot.items)
