@@ -4,7 +4,7 @@ import numpy as np
 from AnyQt.QtWidgets import QGridLayout, QApplication
 
 import Orange.data
-from Orange.data import ContinuousVariable, Domain
+from Orange.data import ContinuousVariable, Domain, Table
 from Orange.widgets.widget import OWWidget, Input, Output, Msg
 from Orange.widgets import gui, settings
 
@@ -110,6 +110,7 @@ class OWFFT(OWWidget):
         self.reader = None
         if self.dx_HeNe is True:
             self.dx = 1.0 / self.laser_wavenumber / 2.0
+        self.use_polar_FFT = False
 
         layout = QGridLayout()
         layout.setContentsMargins(0, 0, 0, 0)
@@ -353,7 +354,10 @@ class OWFFT(OWWidget):
     @gui.deferred
     def commit(self):
         if self.data is not None:
-            self.calculateFFT()
+            if self.use_polar_FFT:
+                self.calculate_polar_FFT()
+            else:
+                self.calculateFFT()
 
     def calculateFFT(self):
         """
@@ -413,32 +417,8 @@ class OWFFT(OWWidget):
                 phase_res=self.phase_resolution if self.phase_res_limit else None,
                 phase_corr=self.phase_corr,
                 peak_search=self.peak_search,
-                )
-                
-        if self.reader == 'NeaReaderGSF':
-            fft_single = irfft.ComplexFFT(
-                    dx=self.dx,
-                    apod_func=self.apod_func,
-                    zff=2**self.zff,
-                    phase_res=self.phase_resolution if self.phase_res_limit else None,
-                    phase_corr=self.phase_corr,
-                    peak_search=self.peak_search,
-                    )
-            full_data = self.data.X[::2] * np.exp(self.data.X[1::2]* 1j)
-            for row in full_data:
-                spectrum_out, phase_out, wavenumbers = fft_single(
-                    row, zpd=stored_zpd_fwd)
-                spectra.append(spectrum_out)
-                spectra.append(phase_out)
-            spectra = np.vstack(spectra)
+            )
 
-            if self.limit_output is True:
-                wavenumbers, spectra = self.limit_range(wavenumbers, spectra)
-            self.spectra_table = build_spec_table(wavenumbers, spectra,
-                                                  additional_table=self.data)
-            self.Outputs.spectra.send(self.spectra_table)
-            return
-            
         for row in ifg_data:
             if self.sweeps in [2, 3]:
                 # split double-sweep for forward/backward
@@ -517,6 +497,55 @@ class OWFFT(OWWidget):
         self.Outputs.spectra.send(self.spectra_table)
         self.Outputs.phases.send(self.phases_table)
 
+    def calculate_polar_FFT(self):
+        # polar channel data comes in alternating channel pairs
+        # of amplitude and phase data for the same channel for each run
+        amplitude_in = self.data.X[::2]
+        phases_in = self.data.X[1::2]
+        ifg_data = amplitude_in * np.exp(phases_in * 1j)
+
+        wavenumbers = None
+        spectra = []
+        phases = []
+
+        # Reset info, error and warning dialogs
+        self.Error.clear()
+        self.Warning.clear()
+
+        fft_single = irfft.ComplexFFT(
+            dx=self.dx,
+            apod_func=self.apod_func,
+            zff=2**self.zff,
+            phase_res=self.phase_resolution if self.phase_res_limit else None,
+            phase_corr=self.phase_corr,
+            peak_search=self.peak_search,
+        )
+        for row in ifg_data:
+            spectrum_out, phase_out, wavenumbers = fft_single(row, zpd=None)
+            spectra.append(spectrum_out)
+            phases.append(phase_out)
+
+        spectra = np.vstack(spectra)
+        phases = np.vstack(phases)
+
+        if self.limit_output is True:
+            wavenumbers, spectra = self.limit_range(wavenumbers, spectra)
+            _, phases = self.limit_range(wavenumbers, phases)
+
+        wavenumbers_domain = Domain(
+            [ContinuousVariable.make(f"{w}") for w in wavenumbers],
+            metas=self.data.domain.metas,
+        )
+        self.spectra_table = Table.from_numpy(
+            wavenumbers_domain, X=spectra, metas=self.data.metas[::2]
+        )
+        phases_table = Table.from_numpy(
+            wavenumbers_domain, X=phases, metas=self.data.metas[1::2]
+        )
+
+        self.Outputs.spectra.send(self.spectra_table)
+        self.Outputs.phases.send(phases_table)
+
     def determine_sweeps(self):
         """
         Determine if input interferogram is single-sweep or
@@ -561,14 +590,25 @@ class OWFFT(OWWidget):
             self.sweeps = 2
 
     def check_metadata(self):
-        """ Look for laser wavenumber and sampling interval metadata """
+        """Look for laser wavenumber and sampling interval metadata"""
 
         try:
-            self.reader = self.data.attributes['Reader']
+            channel_data, detail = self.data.attributes["Channel Data Type"]
+            if channel_data == "Polar":
+                self.use_polar_FFT = True
+                self.infoc.setText(
+                    f"Channel data type: {channel_data} ({detail}).\n"
+                    + "Applying Complex Fourier Transform."
+                )
         except KeyError:
-            self.reader = None
-
-        if self.reader == 'NeaReaderGSF': # TODO Avoid the magic word
+            pass
+        try:
+            domain_units, dx = self.data.attributes["Calculated Datapoint Spacing (Δx)"]
+            if domain_units != "[cm]" or not dx:
+                raise KeyError
+            
+            self.dx = dx
+            self.zff = 2
             self.dx_HeNe = False
             self.dx_HeNe_cb.setDisabled(True)
             self.dx_edit.setDisabled(True)
@@ -581,17 +621,14 @@ class OWFFT(OWWidget):
             self.controls.phase_res_limit.setDisabled(True)
             self.controls.phase_resolution.setDisabled(True)
 
-            info = self.data.attributes
-            number_of_points = int(info['Pixel Area (X, Y, Z)'][3])
-            scan_size = float(info['Interferometer Center/Distance'][2].replace(',', '')) #Microns
-            scan_size = scan_size*1e-4 #Convert to cm
-            step_size = (scan_size * 2) / (number_of_points - 1)
-
-            self.dx = step_size
-            self.zff = 2 #Because is power of 2
-
-            self.infoc.setText(f"Using an automatic datapoint spacing (Δx).\nΔx:\t{self.dx:.8} cm\nApplying Complex Fourier Transform.")
+            self.infoc.setText(
+                self.infoc.text()
+                + "\n"
+                + "Using Calculated Datapoint Spacing (Δx) from metadata."
+            )
             return
+        except KeyError:
+            pass
 
         try:
             lwn = self.data.get_column("Effective Laser Wavenumber")
